@@ -1,17 +1,108 @@
 import sqlite3
 import sys
-from collections.abc import Hashable
 from typing import AbstractSet, Any, Callable, Optional, Union, cast
 from uuid import uuid4
 
-from . import RebuildStrategy
-from .base import _S, _T, SqliteCollectionBase, T
-
 if sys.version_info >= (3, 9):
     from collections.abc import Iterable, Iterator, MutableSet
 else:
     from typing import Iterable, Iterator, MutableSet
 
+from . import RebuildStrategy
+from .base import _S, _T, SqliteCollectionBase, T, TemporaryTableContext, is_hashable
+
+
+class _SetDatabaseDriver:
+    def __init__(self, table_name: str):
+        self.table_name = table_name
+
+    def delete_all(self, cur: sqlite3.Cursor) -> None:
+        cur.execute(f"DELETE FROM {self.table_name}")
+
+    def insert(self, cur: sqlite3.Cursor, serialized_value: bytes) -> None:
+        cur.execute(
+            f"INSERT INTO {self.table_name} (serialized_value) VALUES (?)",
+            (serialized_value,),
+        )
+
+    def upsert(self, cur: sqlite3.Cursor, serialized_value: bytes) -> None:
+        if not self.is_serialized_value_in(cur, serialized_value):
+            self.insert(cur, serialized_value)
+
+    def delete_by_serialized_value(self, cur: sqlite3.Cursor, serialized_value: bytes) -> None:
+        cur.execute(f"DELETE FROM {self.table_name} WHERE serialized_value = ?", (serialized_value,))
+
+    def is_serialized_value_in(self, cur: sqlite3.Cursor, serialized_value: bytes) -> bool:
+        cur.execute(f"SELECT 1 FROM {self.table_name} WHERE serialized_value=?", (serialized_value,))
+        return len(list(cur)) > 0
+
+    def get_one_serialized_value(self, cur: sqlite3.Cursor) -> Union[None, bytes]:
+        cur.execute(f"SELECT serialized_value FROM {self.table_name} LIMIT 1")
+        res = cur.fetchone()
+        if res is None:
+            return None
+        return cast(bytes, res[0])
+
+    def get_count(self, cur: sqlite3.Cursor) -> int:
+        cur.execute(f"SELECT COUNT(*) FROM {self.table_name}")
+        res = cur.fetchone()
+        return cast(int, res[0])
+
+    def get_serialized_values(self, cur: sqlite3.Cursor) -> Iterable[bytes]:
+        cur.execute(f"SELECT serialized_value FROM {self.table_name}")
+        for d in cur:
+            yield cast(bytes, d[0])
+
+    def intersection_update_single(self, cur: sqlite3.Cursor, data: Iterable[bytes]) -> None:
+        with TemporaryTableContext(cur, self.table_name) as temp_table_name:
+            temporay_driver = _SetDatabaseDriver(temp_table_name)
+            for d in data:
+                temporay_driver.upsert(cur, d)
+            cur.execute(
+                f"DELETE FROM {self.table_name} WHERE NOT EXISTS (SELECT serialized_value FROM {temp_table_name} WHERE {self.table_name}.serialized_value = {temp_table_name}.serialized_value)"
+            )
+
+    def difference_update_single(self, cur: sqlite3.Cursor, data: Iterable[bytes]) -> None:
+        for d in data:
+            self.delete_by_serialized_value(cur, d)
+
+    def union_update_single(self, cur: sqlite3.Cursor, data: Iterable[bytes]) -> None:
+        for d in data:
+            self.upsert(cur, d)
+
+    def symmetric_difference_update_single(
+        self, cur: sqlite3.Cursor, cur2: sqlite3.Cursor, data: Iterable[bytes]
+    ) -> None:
+        with TemporaryTableContext(cur, self.table_name) as temp_table_name:
+            temporay_driver = _SetDatabaseDriver(temp_table_name)
+            for d in data:
+                temporay_driver.upsert(cur, d)
+            for serialized_value in temporay_driver.get_serialized_values(cur2):
+                if self.is_serialized_value_in(cur, serialized_value):
+                    self.delete_by_serialized_value(cur, serialized_value)
+                else:
+                    self.insert(cur, serialized_value)
+
+    def is_proper_superset(self, cur: sqlite3.Cursor, cur2: sqlite3.Cursor, data: Iterable[bytes]) -> bool:
+        with TemporaryTableContext(cur, self.table_name) as temp_table_name:
+            temporay_driver = _SetDatabaseDriver(temp_table_name)
+            for d in data:
+                if not self.is_serialized_value_in(cur2, d):
+                    return False
+                temporay_driver.upsert(cur2, d)
+            return temporay_driver.get_count(cur2) < self.get_count(cur2)
+
+    def is_proper_subset(self, cur: sqlite3.Cursor, cur2: sqlite3.Cursor, data: Iterable[bytes]) -> bool:
+        is_proper = False
+        with TemporaryTableContext(cur, self.table_name) as temp_table_name:
+            temporary_driver = _SetDatabaseDriver(temp_table_name)
+            for d in data:
+                if not self.is_serialized_value_in(cur2, d):
+                    is_proper = True
+                else:
+                    temporary_driver.upsert(cur2, d)
+            return is_proper and temporary_driver.get_count(cur2) == self.get_count(cur2)
+
 
 class Set(SqliteCollectionBase[T], MutableSet[T]):
     def __init__(
@@ -33,6 +124,7 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
             rebuild_strategy=rebuild_strategy,
             do_initialize=True,
         )
+        self._database_driver = _SetDatabaseDriver(self.table_name)
         if data is not None:
             self.clear()
             self.update(data)
@@ -40,21 +132,16 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
     def __contains__(self, value: object) -> bool:
         cur = self.connection.cursor()
         serialized_value = self.serialize(cast(T, value))
-        return self._is_serialized_value_in(cur, serialized_value)
+        return self._database_driver.is_serialized_value_in(cur, serialized_value)
 
     def __iter__(self) -> Iterator[T]:
         cur = self.connection.cursor()
-        for d in self._get_serialized_values(cur):
+        for d in self._database_driver.get_serialized_values(cur):
             yield self.deserialize(d)
 
-    def _get_serialized_values(self, cur: sqlite3.Cursor) -> Iterable[bytes]:
-        cur.execute(f"SELECT serialized_value FROM {self.table_name}")
-        for d in cur:
-            yield cast(bytes, d[0])
-
     def __len__(self) -> int:
         cur = self.connection.cursor()
-        return self._get_count(cur)
+        return self._database_driver.get_count(cur)
 
     def _do_create_table(self, commit: bool = False) -> None:
         cur = self.connection.cursor()
@@ -92,88 +179,57 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
         return serialized_value != self.serialize(value)
 
     def serialize(self, value: T) -> bytes:
-        if not self._is_hashable(value):
+        if not is_hashable(value):
             raise TypeError(f"unhashable type: '{type(value).__name__}'")
         return self.serializer(value)
 
     def add(self, value: T) -> None:
         serialized_value = self.serialize(value)
         cur = self.connection.cursor()
-        self._upsert(cur, serialized_value)
+        self._database_driver.upsert(cur, serialized_value)
         self.connection.commit()
 
     def clear(self) -> None:
         cur = self.connection.cursor()
-        self._delete_all(cur)
+        self._database_driver.delete_all(cur)
         self.connection.commit()
 
-    def _delete_all(self, cur: sqlite3.Cursor) -> None:
-        cur.execute(f"DELETE FROM {self.table_name}")
-
-    def _upsert(self, cur: sqlite3.Cursor, serialized_value: bytes) -> None:
-        if not self._is_serialized_value_in(cur, serialized_value):
-            cur.execute(
-                f"INSERT INTO {self.table_name} (serialized_value) VALUES (?)",
-                (serialized_value,),
-            )
-
-    def _delete_by_serialized_value(self, cur: sqlite3.Cursor, serialized_value: bytes) -> None:
-        cur.execute(f"DELETE FROM {self.table_name} WHERE serialized_value = ?", (serialized_value,))
-
-    def _is_serialized_value_in(self, cur: sqlite3.Cursor, serialized_value: bytes) -> bool:
-        cur.execute(f"SELECT 1 FROM {self.table_name} WHERE serialized_value=?", (serialized_value,))
-        return len(list(cur)) > 0
-
     def discard(self, value: T) -> None:
         cur = self.connection.cursor()
-        self._delete_by_serialized_value(cur, self.serialize(value))
+        self._database_driver.delete_by_serialized_value(cur, self.serialize(value))
         self.connection.commit()
 
     def remove(self, value: T) -> None:
         cur = self.connection.cursor()
         serialized_value = self.serialize(value)
-        if not self._is_serialized_value_in(cur, serialized_value):
+        if not self._database_driver.is_serialized_value_in(cur, serialized_value):
             raise KeyError(value)
-        self._delete_by_serialized_value(cur, serialized_value)
+        self._database_driver.delete_by_serialized_value(cur, serialized_value)
         self.connection.commit()
 
     def pop(self) -> T:
         cur = self.connection.cursor()
-        serialized_value = self._get_one_serialized_value(cur)
+        serialized_value = self._database_driver.get_one_serialized_value(cur)
         if serialized_value is None:
             raise KeyError("'pop from an empty set'")
-        self._delete_by_serialized_value(cur, serialized_value)
+        self._database_driver.delete_by_serialized_value(cur, serialized_value)
         self.connection.commit()
         return self.deserialize(serialized_value)
 
-    def _get_one_serialized_value(self, cur: sqlite3.Cursor) -> Union[None, bytes]:
-        cur.execute(f"SELECT serialized_value FROM {self.table_name} LIMIT 1")
-        res = cur.fetchone()
-        if res is None:
-            return None
-        return cast(bytes, res[0])
-
     @property
     def schema_version(self) -> str:
         return "0"
 
-    def _is_hashable(self, value: T) -> bool:
-        return isinstance(value, Hashable)
-
-    def _get_count(self, cur: sqlite3.Cursor) -> int:
-        cur.execute(f"SELECT COUNT(*) FROM {self.table_name}")
-        res = cur.fetchone()
-        return cast(int, res[0])
-
     def issubset(self, other: Iterable[T]) -> bool:
         return len(self) == len(self.intersection(other))
 
-    def _intersection_single(self, other: Iterable[T]) -> "Set[T]":
-        res = self._create_volatile_copy([])
-        for d in other:
-            if d in self:
-                res.add(d)
-        return res
+    def __lt__(self, other: Iterable[T]) -> bool:
+        return self._database_driver.is_proper_subset(
+            self.connection.cursor(), self.connection.cursor(), (self.serialize(d) for d in other)
+        )
+
+    def __le__(self, other: Iterable[T]) -> bool:
+        return self.issubset(other)
 
     def intersection(self, *others: Iterable[T]) -> "Set[T]":
         res = self.copy()
@@ -181,23 +237,26 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
         return res
 
     def intersection_update(self, *others: Iterable[T]) -> None:
+        cur = self.connection.cursor()
         for other in others:
-            self._intersection_update_single(other)
+            self._database_driver.intersection_update_single(cur, (self.serialize(d) for d in other))
         self.connection.commit()
 
-    def _intersection_update_single(self, other: Iterable[T]) -> None:
-        buf = self._create_volatile_copy(other)
-        cur = self.connection.cursor()
-        cur.execute(
-            f"DELETE FROM {self.table_name} WHERE NOT EXISTS (SELECT serialized_value FROM {buf.table_name} WHERE {self.table_name}.serialized_value = {buf.table_name}.serialized_value)"
-        )
-
     def issuperset(self, other: Iterable[T]) -> bool:
+        cur = self.connection.cursor()
         for d in other:
-            if not d in self:
+            if not self._database_driver.is_serialized_value_in(cur, self.serialize(d)):
                 return False
         return True
 
+    def __gt__(self, other: Iterable[T]) -> bool:
+        return self._database_driver.is_proper_superset(
+            self.connection.cursor(), self.connection.cursor(), (self.serialize(d) for d in other)
+        )
+
+    def __ge__(self, other: Iterable[T]) -> bool:
+        return self.issuperset(other)
+
     def union(self, *others: Iterable[T]) -> "Set[T]":
         res = self.copy()
         res.update(*others)
@@ -206,12 +265,15 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
     def update(self, *others: Iterable[T]) -> None:
         cur = self.connection.cursor()
         for other in others:
-            self._union_update_single(cur, other)
+            self._database_driver.union_update_single(cur, (self.serialize(d) for d in other))
         self.connection.commit()
 
-    def _union_update_single(self, cur: sqlite3.Cursor, other: Iterable[T]) -> None:
+    def isdisjoint(self, other: Iterable[T]) -> bool:
+        cur = self.connection.cursor()
         for d in other:
-            self._upsert(cur, self.serialize(d))
+            if self._database_driver.is_serialized_value_in(cur, self.serialize(d)):
+                return False
+        return True
 
     @classmethod
     def _from_iterable(cls, *args: T) -> "Set[T]":
@@ -231,13 +293,9 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
     def difference_update(self, *others: Iterable[T]) -> None:
         cur = self.connection.cursor()
         for other in others:
-            self._difference_update_single(cur, other)
+            self._database_driver.difference_update_single(cur, (self.serialize(d) for d in other))
         self.connection.commit()
 
-    def _difference_update_single(self, cur: sqlite3.Cursor, other: Iterable[T]) -> None:
-        for d in other:
-            self._delete_by_serialized_value(cur, self.serialize(d))
-
     def _create_volatile_copy(self, data: Optional[Iterable[T]] = None) -> "Set[T]":
         return Set[T](
             connection=self.connection,
@@ -261,19 +319,11 @@ class Set(SqliteCollectionBase[T], MutableSet[T]):
 
     def symmetric_difference_update(self, *others: Iterable[T]) -> None:
         cur = self.connection.cursor()
+        cur2 = self.connection.cursor()
         for other in others:
-            self._symmetric_difference_update_single(cur, other)
+            self._database_driver.symmetric_difference_update_single(cur, cur2, (self.serialize(d) for d in other))
         self.connection.commit()
 
-    def _symmetric_difference_update_single(self, cur: sqlite3.Cursor, other: Iterable[T]) -> None:
-        buf = self._create_volatile_copy(other)
-        iter_cursor = buf.connection.cursor()
-        for serialized_value in buf._get_serialized_values(iter_cursor):
-            if self._is_serialized_value_in(cur, serialized_value):
-                self._delete_by_serialized_value(cur, serialized_value)
-            else:
-                self._upsert(cur, serialized_value)
-
     def __xor__(self, s: AbstractSet[_T]) -> "Set[T]":
         return self.symmetric_difference(cast(Iterable[T], s))
 
